@@ -4,7 +4,7 @@ package test;
 
 =head1 NAME
 
-test - tools for helping in test suites
+test - tools for helping in test suites (not including running external programs).
 
 =head1 SYNOPSIS
 
@@ -22,21 +22,6 @@ test - tools for helping in test suites
          ;
   }
 
-  {
-    my $outcount = 1;
-    my ($out, $err) = '';
-    my $teststring = "\n__FOO__\n\n";
-    ok runcheck
-        ( [[':psreplace',
-            -v => 'TEST=Cholet', -D => 'TEST', ],
-           '<', \$teststring,
-           '>', \$out, '2>', \$err],
-          "psreplace -D",
-          \$err,
-        ), 1, 'runcheck -D';
-    ok $out, "\nCholet\n\n", 'outputcheck -D';
-  }
-
   ok evcheck(sub {
                open my $fh, '>', 'foo';
                print $fh "$_\n"
@@ -53,6 +38,10 @@ test - tools for helping in test suites
 This package provides some variables, and sets up an environment, for test
 scripts, such as those used in F<t/>.
 
+This package does not including running external programs; that is provided by
+C<test2.pm>.  This is so that suites not needing that can include only
+test.pm, and so not require the presence of C<IPC::Run>.
+
 Setting up the environment includes:
 
 =over 4
@@ -82,9 +71,9 @@ Unless TEST_DEBUG is set in the environment.
 
 # Pragmas -----------------------------
 
-use v5.6.1;
+use 5.00503;
 use strict;
-use warnings;
+use vars qw( @EXPORT_OK );
 
 # Inheritance -------------------------
 
@@ -104,40 +93,66 @@ The following symbols are exported upon request:
 
 =item LIB_DIR
 
-=item only_files
+=item check_req
 
-=item runcheck
-
-=item simple_run_test
+=item compare
 
 =item evcheck
 
+=item only_files
+
+=item save_output
+
+=item restore_output
+
 =item tmpnam
 
+=item tempdir
+
 =back
 
 =cut
 
-our @EXPORT_OK = qw( BIN_DIR DATA_DIR REF_DIR LIB_DIR
-                     only_files runcheck simple_run_test evcheck
-                     save_output restore_output tmpnam );
+@EXPORT_OK = qw( BIN_DIR DATA_DIR REF_DIR LIB_DIR
+                 check_req compare evcheck only_files
+                 save_output restore_output tempdir tmpnam );
 
 # Utility -----------------------------
 
 use Carp                          qw( carp croak );
-use Env                           qw( @PERL5LIB );
-use Fatal                    1.02 qw( :void close open seek sysopen unlink );
-use Fcntl                    1.03 qw( :DEFAULT :seek );
-use File::Compare          1.1002 qw( compare );
-use File::Path             1.0403 qw( mkpath rmtree );
-use File::Spec::Functions         qw( catdir catfile updir );
-use File::Temp               0.12 qw( tempfile tempdir );
+use Cwd                      2.01 qw( cwd );
+use Env                           qw( PERL5LIB );
+use Fatal                    1.02 qw( close open seek sysopen unlink );
+use Fcntl                    1.03 qw( :DEFAULT );
+use File::Compare          1.1002 qw( );
+use File::Path             1.0401 qw( mkpath rmtree );
+use File::Spec                0.6 qw( );
 use FindBin                  1.42 qw( $Bin );
-use IPC::Run                 0.44 qw( harness run );
-use Test                     1.13 qw( ok );
+use POSIX                    1.02 qw( );
+use Test                    1.122 qw( ok skip );
 
 # ----------------------------------------------------------------------------
 
+sub rel2abs {
+  if ( File::Spec->file_name_is_absolute($_[0]) ) {
+    return $_[0];
+  } else {
+    return catdir(cwd, $_[0]);
+  }
+}
+
+sub catdir {
+  File::Spec->catdir(@_);
+}
+
+sub catfile {
+  File::Spec->catfile(@_);
+}
+
+sub updir {
+  File::Spec->updir(@_);
+}
+
 # -------------------------------------
 # PACKAGE CONSTANTS
 # -------------------------------------
@@ -151,196 +166,28 @@ use constant LIB_DIR  => catdir $Bin, updir, 'lib';
 # PACKAGE ACTIONS
 # -------------------------------------
 
-unshift @PERL5LIB, LIB_DIR;
+# @PERL5LIB not available in Env for perl 5.00503
+# unshift @PERL5LIB, LIB_DIR;
+$PERL5LIB = join ':', LIB_DIR, $PERL5LIB;
 unshift @INC,      LIB_DIR;
 
-my $tmpdn = tempdir(CLEANUP => ( ! defined $ENV{TEST_DEBUG}     or
-                                 $ENV{TEST_DEBUG} !~ /\bSAVE\b/ ));
+$_ = rel2abs($_)
+  for @INC;
+
+my $tmpdn = tempdir();
+$| = 1;
+
 mkpath $tmpdn;
 die "Couldn't create temp dir: $tmpdn: $!\n"
   unless -r $tmpdn and -w $tmpdn and -x $tmpdn and -o $tmpdn and -d $tmpdn;
-chdir $tmpdn;
 
-END {
-  print STDERR "Used tmp dir: $tmpdn\n"
-    if defined $ENV{TEST_DEBUG} and $ENV{TEST_DEBUG} =~ /\bSAVE\b/;
-}
+#@INC = map rel2abs($_), @INC;
+chdir $tmpdn;
 
 # -------------------------------------
 # PACKAGE FUNCTIONS
 # -------------------------------------
 
-=head2 runcheck
-
-Run an external command, check the results.
-
-=over 4
-
-=item ARGUMENTS
-
-=over 4
-
-=item runargs
-
-An arrayref of arguments as for L<IPC::Run/run>, excepting that array ref
-arguments with an initial C<:> character on the first member will be
-considered as perl scripts in the module built to run.
-
-For example, an invocation of
-
-  runcheck([':reverse'], '<', '/etc/passwd');
-
-will convert the initial reverse to treat it as a perl script called
-F<reverse> to find in the module, and execute that with the current running
-perl.  The remaining arguments are left as is.
-
-=item name
-
-The name of the program to refer to in error messages
-
-=item errref
-
-Reference to a scalar to read in case of error.  Normally, this is bound to a
-scalar where is deposited the stderr out of the command, using arguments
-
-  '2>', $err
-
-in L</runargs>.
-
-=back
-
-=item RETURNS
-
-=over 4
-
-=item success
-
-1 if the command executed without failure; false otherwise.
-
-=back
-
-=back
-
-=cut
-
-sub runcheck {
-  my ($runargs, $name, $errref) = @_;
-
-  my @args = map({ ( ref $_ eq 'ARRAY' and substr($_->[0],0,1) eq ':') ?
-                     [ $^X, catfile(BIN_DIR, substr($_->[0],1)),
-                       @{$_}[1..$#$_] ]                                :
-                     $_ }
-                 @$runargs);
-  my $harness = harness(@args);
-  run $harness;
-  my $rv = $harness->full_result;
-  if ( $rv ) {
-    if ( $ENV{TEST_DEBUG} ) {
-      warn sprintf("$name failed: exit/sig/core %d/%d/%d\n",
-                   $rv >> 8, $rv & 127, ( $rv & 128 ) >> 7);
-      warn "  $$errref\n"
-        if defined $errref and defined $$errref and $$errref !~ /^\s*$/;
-    }
-    return;
-  } else {
-    return 1;
-  }
-}
-
-# -------------------------------------
-
-=head2 simple_run_test
-
-This is designed to simplify the job of running a program, and testing the
-output.  It performs 2+n tests; that the command executed without error, that
-the n files named in the C<checkfiles> argument are each as expected, and that
-no other files exist.
-
-All files in the current directory are wiped after the test in preparation for
-the next test.
-
-=over 4
-
-=item ARGUMENTS
-
-The arguments are considered as name/value pairs.
-
-=over 4
-
-=item runargs
-
-B<Mandatory>.  This is an arrayref; as for the runargs argument to
-L<runcheck|/runcheck>.
-
-=item name
-
-B<Mandatory>.  The name to use in error messages.
-
-=item checkfiles
-
-This is an arrayref of files to check.  The named files are considered
-relative to the working directory, and are checked against files taken
-relative to the F<testref> directory of the build.  Therefore, absolute file
-names are non-sensical, and will raise an exception.
-
-=item errref
-
-A ref to a scalar potentially containing any error output.  Typically, the
-stderr of the command run is redirected to this by the runargs argument.
-
-=item testref_subdir
-
-A subdirectory of the testref directory in which to find the files to check
-against.
-
-=back
-
-=item RETURNS
-
-I<None>
-
-However, 2+n tests are performed, with ok/not ok sent to stdout.
-
-=back
-
-=cut
-
-sub simple_run_test {
-  my (%arg) = @_;
-
-  die sprintf("%s: missing mandatory argument: %s\n", (caller(0))[3], $_)
-    for grep ! exists $arg{$_}, qw( runargs name );
-
-  ${$arg{errref}} = ''
-    if exists $arg{errref};
-  my $runok = runcheck(@arg{qw(runargs name errref)});
-
-  ok $runok, 1, $arg{name};
-
-  my $ref_dir = (exists $arg{testref_subdir}           ?
-                 catdir(REF_DIR, $arg{testref_subdir}) :
-                 REF_DIR);
-
-  if ( exists $arg{checkfiles} ) {
-    for (@{$arg{checkfiles}}) {
-      my $target = catfile($ref_dir, $_);
-      if ( -e $target ) {
-        ok(compare($_, $target), 0, "$arg{name}: check file $_");
-      } else {
-        ok 0, 1, "$arg{name}: missing reference file $_";
-      }
-    }
-  }
-
-  ok(only_files($arg{checkfiles}), 1, "$arg{name}: no extra files");
-  # Clean up files for next test.
-  opendir my $dir, '.';
-  unlink grep !/^\.\.?$/, readdir $dir;
-  closedir $dir;
-}
-
-# -------------------------------------
-
 =head2 only_files
 
 =over 4
@@ -372,16 +219,17 @@ Arrayref of names of files to expect to exist.
 sub only_files {
   my ($expect) = @_;
 
-  opendir my $dir, '.';
-  my %files = map { $_ => 1 } readdir $dir;
-  closedir $dir;
+  local *MYDIR;
+  opendir MYDIR, '.';
+  my %files = map { $_ => 1 } readdir MYDIR;
+  closedir MYDIR;
 
   my $ok = 1;
 
   for (@$expect, '.', '..') {
     if ( exists $files{$_} ) {
       delete $files{$_};
-    } else {
+    } elsif ( ! -e $_ ) { # $_ might be absolute
       carp "File not found: $_\n"
         if $ENV{TEST_DEBUG};
       $ok = 0;
@@ -492,7 +340,9 @@ sub save_output {
     unless @_ == 2;
   my ($name, $filehandle) = @_;
 
-  my $tmpfh = tempfile;
+  my $tmpnam = POSIX::tmpnam;
+  sysopen my $tmpfh, $tmpnam, O_RDWR | O_CREAT | O_EXCL;
+  unlink $tmpnam;
   select((select($tmpfh), $| = 1)[0]);
 
   open my $savefh, '>&' . fileno $filehandle
@@ -547,7 +397,7 @@ sub restore_output {
   open  $origfh, '>&' . fileno $savefh
     or die "cannot dup $name back again: $!";
 
-  seek $tmpfh, 0, SEEK_SET;
+  seek $tmpfh, 0, 0;
   local $/ = undef;
   my $string = <$tmpfh>;
   close $tmpfh;
@@ -574,15 +424,22 @@ if TEST_DEBUG has SAVE in the value.
 
 =item ARGUMENTS
 
-I<None>
+=over 4
+
+=item name
+
+I<Optional>.  If defined, a name by which to refer to the tmpfile in user
+messages.
+
+=back
 
 =item RETURNS
 
 =over 4
 
-=item name
+=item filename
 
-Name of temporary file name.
+Name of temporary file.
 
 =item fh
 
@@ -597,7 +454,7 @@ context.
 
 my @tmpfns;
 sub tmpnam {
-  my $tmpnam = File::Temp::tmpnam;
+  my $tmpnam = POSIX::tmpnam;
 
   if (@_) {
     push @tmpfns, [ $tmpnam, $_[0] ];
@@ -623,10 +480,281 @@ END {
       }
     }
   } else {
-    unlink map((ref $_ ? $_->[0] : $_), @tmpfns);
+    unlink map((ref $_ ? $_->[0] : $_), @tmpfns)
+      if @tmpfns;
   }
 }
 
+# -------------------------------------
+
+=head2 tempdir
+
+Very much like the one in L<POSIX> or L<File::Temp>, but does not get deleted
+if TEST_DEBUG has SAVE in the value (does get deleted otherwise).
+
+=over 4
+
+=item ARGUMENTS
+
+I<None>
+
+=item RETURNS
+
+=over 4
+
+=item name
+
+Name of temporary dir.
+
+=back
+
+=back
+
+=cut
+
+my @tmpdirs;
+sub tempdir {
+  my $tempdir = POSIX::tmpnam;
+  mkdir $tempdir, 0700
+    or die "Failed to create temporary directory $tempdir: $!\n";
+
+  if (@_) {
+    push @tmpdirs, [ $tempdir, $_[0] ];
+  } else {
+    push @tmpdirs, $tempdir;
+  }
+
+  return $tempdir;
+}
+
+END {
+  for (@tmpdirs) {
+    if ( ref $_ ) {
+      if ( defined $ENV{TEST_DEBUG} and $ENV{TEST_DEBUG} =~ /\bSAVE\b/ ) {
+        printf "Used temp dir: %s (%s)\n", @$_;
+      } else {
+        rmtree $_->[0];
+      }
+    } else {
+      if ( defined $ENV{TEST_DEBUG} and $ENV{TEST_DEBUG} =~ /\bSAVE\b/ ) {
+        print "Used temp dir: $_\n";
+      } else {
+        rmtree $_;
+      }
+    }
+  }
+}
+
+# -------------------------------------
+
+=head2 compare
+
+  ok compare($fn1, $fn2), 1, 'test name';
+
+Compare two files, return 1 if they are the same, 0 if they differ, -1 if file
+cannot be read, -2 if file 2 cannot be read, -4 if both cannot be read.  -8
+for a usage error.
+
+In TEST_DEBUG mode, if the files do not compare, outputs file info on STDERR.
+
+=cut
+
+sub compare {
+  my ($fn1, $fn2) = @_;
+
+  for ( grep ! defined, $fn1, $fn2 ) {
+    carp "Usage: compare($fn1, $fn2)\n"
+        if $ENV{TEST_DEBUG};
+    return -8;
+  }
+
+  {
+    my $err = 0;
+
+    for (0..1) {
+      my $fn = ($fn1, $fn2)[$_];
+      if ( ! -e $fn ) {
+        carp "Does not exist: $fn\n"
+          if $ENV{TEST_DEBUG};
+        $err |= 2 ** $_;
+      } elsif ( ! -r $fn ) {
+        carp "Cannot read: $fn\n"
+          if $ENV{TEST_DEBUG};
+        $err |= 2 ** $_;
+      }
+    }
+
+    return -$err
+      if $err;
+  }
+
+  return 1
+    unless File::Compare::compare($fn1, $fn2);
+
+  if ( $ENV{TEST_DEBUG} ) {
+    my $pid = fork;
+    die "Fork failed: $!\n"
+      unless defined $pid;
+
+    if ( $pid ) { # Parent
+      my $waitpid = waitpid($pid, 0);
+      die "Waitpid got: $waitpid (expected $pid)\n"
+        unless $waitpid == $pid;
+    } else { # Child
+      open *STDOUT{IO}, ">&" . fileno STDERR;
+      exec qw(ls -l), $fn1, $fn2;
+    }
+
+    my $fh1 = IO::File->new($fn1, O_RDONLY)
+      or die "$fn1: $!\n";
+    my $fh2 = IO::File->new($fn2, O_RDONLY)
+      or die "$fn2: $!\n";
+
+    local $/ = "\n";
+
+    my $found = 0;
+    while ( ! $found and my $line1 = <$fh1> ) {
+      my $line2 = <$fh2>;
+      if ( ! defined $line2 ) {
+        print STDERR "$fn2 ended at line: $.\n";
+        $found = 1;
+      } elsif ( $line2 ne $line1 ) {
+        print STDERR
+          "Difference at line $.:\n-1->$line1<---\n-2->$line2<---\n-";
+        $found = 1;
+      }
+    }
+
+    if ( ! $found ) {
+      my $line2 = <$fh2>;
+      if ( defined $line2 ) {
+        print STDERR "$fn1 ended before line: $.\n";
+      } else {
+        print STDERR "Difference between $fn1, $fn2 not found!\n";
+      }
+    }
+
+    close $fh1;
+    close $fh2;
+  }
+
+  return 0;
+}
+
+# -------------------------------------
+
+=head2 check_req
+
+Perform a requisite check on a given executable.  This will skip if the
+required modules are not present.
+
+4+(n+m)*2 tests are performed, where n is the number of prerequisites
+expected, and m is the number of outputs expected.
+
+=over 4
+
+=item SYNOPSIS
+
+  check_req('ccu-touch',
+            ['/etc/passwd'],
+            [[REQ_FILE, '/etc/passwd']],
+            [[REQ_FILE, 'passwd.foo']],
+            'requisites 1');
+
+
+=item ARGUMENTS
+
+=over 4
+
+=item cmd_name
+
+The name of the command to run.  It is assumed that this command is in
+blib/script; hence it should be an executable in this package, and C<make>
+shuold have been run recently.
+
+=item args
+
+The arguments to pass to the cmd_name, as an arrayref.
+
+=item epres
+
+The expected prerequisites, as an arrayref, wherein every member is a
+two-element arrayref, the members being the requisite type, and the requisite
+value.
+
+=item eouts
+
+The expected outputs, in the same format as the L<epres|"epres">.
+
+=item testname
+
+The name to use in error messages.
+
+=back
+
+=back
+
+=cut
+
+sub check_req {
+  my ($cmd_name, $args, $epres, $eouts, $testname) = @_;
+
+  eval "use Pipeline::DataFlow 1.03 qw( :req_types );";
+  my $skip;
+  if ( $@ ) {
+    print STDERR "$@\n"
+      if $ENV{TEST_DEBUG};
+    $skip = 'Skipped: Pipeline::DataFlow 1.03 not found';
+  } else {
+    $skip = 0;
+  }
+
+  my $count = 1;
+  my $test = sub {
+    my ($code, $expect) = @_;
+    my $name = sprintf "%s (%2d)", $testname, $count++;
+    my $value = UNIVERSAL::isa($code, 'CODE') ? $code->($name) : $code;
+    skip $skip, $value, $expect, $name;
+  };
+
+  # Initialize nicely to cope when read_reqs fails
+  my ($pres, $outs) = ([], []);
+
+  $test->(sub {
+            evcheck(sub {
+                      ($pres, $outs) = Pipeline::DataFlow->read_reqs
+                        ([catfile($Bin, updir, 'blib', 'script', $cmd_name),
+                          @$args]);
+                    }, $_[0]),},
+          1);
+
+  $test->(scalar @$pres, scalar @$epres);
+
+  my (@epres, @pres);
+  @epres = sort { $a->[1] cmp $b->[1] } @$epres;
+  @pres =  sort { $a->[1] cmp $b->[1] } @$pres;
+
+  for (my $i = 0; $i < @epres; $i++) {
+    my ($type, $value) = @{$epres[$i]};
+    $test->($type,  @pres > $i ? $pres[$i]->[0] : undef);
+    $test->($value, @pres > $i ? $pres[$i]->[1] : undef);
+  }
+
+  $test->(scalar @$outs, scalar @$eouts);
+
+  my (@eouts, @outs);
+  @eouts = sort { $a->[1] cmp $b->[1] } @$eouts;
+  @outs =  sort { $a->[1] cmp $b->[1] } @$outs;
+
+  for (my $i = 0; $i < @eouts; $i++) {
+    my ($type, $value) = @{$eouts[$i]};
+    $test->($type,  @outs > $i ? $outs[$i]->[0] : undef);
+    $test->($value, @outs > $i ? $outs[$i]->[1] : undef);
+  }
+
+  $test->(only_files([]), 1);
+}
+
 # ----------------------------------------------------------------------------
 
 =head1 EXAMPLES
